#lang racket
(require rackunit)
(define left car)
(define right cdr)

(define (pair-swap p)
  (cons (right p) (left p)))

(check-equal?
  (pair-swap (cons 1 2))
  (cons 2 1))

(check-equal?
  (pair-swap (cons 3 1))
  (cons 1 3))

; CamelCaseStyle
; snake_style
; kebab-style

(define (pointwise f l r)
  (cons
    (f (left l) (left r))
    (f (right l) (right r)))
)



(define (pair+ l r)
  (cons
    (+ (left l) (left r))
    (+ (right l) (right r)))
)

(define (pair+:2 l r)
  (pointwise + l r))

(define (pair* l r)
  (pointwise * l r))


(check-equal?
  (cons 4 6)
  (pair+ (cons 1 2) (cons 3 4)))

(check-equal?
  (cons 4 6)
  (pair+:2 (cons 1 2) (cons 3 4)))

(check-equal?
  (cons 3 8)
  (pair* (cons 1 2) (cons 3 4)))


(define (pair< l r)
  (or
    (< (left l) (left r))
    (and (= (left l) (left r))
         (< (right l) (right r)))))

(check-true (pair< (cons 1 3) (cons 2 3)))
(check-true (pair< (cons 1 2) (cons 1 3)))
(check-false (pair< (cons 1 3) (cons 1 3)))
(check-false (pair< (cons 1 3) (cons 1 0)))
(check-false (pair< (cons 10 3) (cons 1 0)))

; ==========================================

(define (point x y z)
  (list x y z))

(define (point? p)
  (and (list? p) (= (length p) 3)))

(point? #f)
(point? 10)

#|
  (cons x
    (cons y
      (cons z empty)))
|#

;; (define (point-x p)
;;   (left p))
;; (define (point-y p)
;;   (left (right p)))
;; (define (point-z p)
;;   (left (right (right p))))

;; (define (point-x p)
;;   (car p))
;; (define (point-y p)
;;   (car (cdr p)))
;; (define (point-z p)
;;   (car (cdr (cdr p))))
;; (define (point-y p)
;;   (cadr p))
;; (define (point-z p)
;;   (caddr p))

(define (point-x p)
  (first p))
(define (point-y p)
  (first (rest p)))
(define (point-z p)
  (first (rest (rest p))))

;; (define (point-x p)
;;   (first p))
;; (define (point-y p)
;;   (second p))
;; (define (point-z p)
;;   (third p))


(define p (point 1 2 3))
(check-true (point? p))
(check-equal? (list 1 2 3) p)
(check-equal? 1 (point-x p))
(check-equal? 2 (point-y p))
(check-equal? 3 (point-z p))
;; (check-true (origin? (list 0 0 0)))
;; (check-false (origin? p))













